#include "GetObjects.js"

function CreateLine( ptA, ptB )
{
	var linefactory = moi.command.createFactory( 'line' );
	linefactory.setInput( 0, ptA );
	linefactory.setInput( 1, ptB );
	var line = linefactory.calculate().item( 0 );
	linefactory.cancel();

	return line;
}

function CreatePoint( x, y, z )
{
	return moi.vectorMath.createPoint( x, y, z );
}

function CreatePointObject( x, y, z )
{
	var ptfactory = moi.command.createFactory( 'point' );
	ptfactory.setInput( 0, CreatePoint( x, y, z ) );
	var pt = ptfactory.calculate().item( 0 );
	ptfactory.cancel();

	return pt;
}

function GetPointAtDistance( crv, full_length, dist )
{
	var flowfactory = moi.command.createFactory( 'flow' );

	var ptlist = moi.geometryDatabase.createObjectList();
	ptlist.addObject( CreatePointObject(dist, 0, 0) );
	flowfactory.setInput( 0, ptlist );

	var baseline = CreateLine( CreatePoint(0,0,0), CreatePoint(full_length,0,0) );
	flowfactory.setInput( 1, baseline );
	flowfactory.setInput( 2, crv );

	var crvpts = flowfactory.calculate();
	flowfactory.cancel();

	var t = crv.dropPoint( crvpts.item(0).pt );
	return crv.evaluatePoint( t );
}

function TrimCurve( crv, p0, p1 )
{
	var trimfactory = moi.command.createFactory( 'trim' );

	var crvlist = moi.geometryDatabase.createObjectList();
	crvlist.addObject( crv );
	trimfactory.setInput( 0, crvlist );

	var ptlist = moi.geometryDatabase.createObjectList();
	ptlist.addObject( CreatePointObject(p0.x,p0.y,p0.z) );
	ptlist.addObject( CreatePointObject(p1.x,p1.y,p1.z) );
	trimfactory.setInput( 1, ptlist );

	trimfactory.setInput( 2, moi.geometryDatabase.createObjectList() );
	trimfactory.generateFragments();
	trimfactory.finishedPickingFragments();

	var pieces = trimfactory.calculate();
	trimfactory.cancel();

	var tol = 0.00001;

	for ( var i = 0; i < pieces.length; ++i )
	{
		var piece = pieces.item(i);

		var dist00 = moi.vectorMath.distance( piece.getStartPt(), p0 );
		var dist11 = moi.vectorMath.distance( piece.getEndPt(), p1 );
		var dist10 = moi.vectorMath.distance( piece.getEndPt(), p0 );
		var dist01 = moi.vectorMath.distance( piece.getStartPt(), p1 );

		if ( (dist00 < tol && dist11 < tol) || (dist10 < tol && dist01 < tol) )
			return piece;
	}

	return null;
}

var g_Pieces = null;

function Update( curves )
{
	var DashSize = moi.ui.commandUI.DashSize.value;
	var GapSize = moi.ui.commandUI.GapSize.value;
	if ( DashSize < 1.0e-6 || GapSize < 0.0 )
		return;

	if ( g_Pieces )
	{
		moi.geometryDatabase.removeObjects( g_Pieces );
		g_Pieces = null;
	}

	g_Pieces = moi.geometryDatabase.createObjectList();

	for ( var i = 0; i < curves.length; ++i )
	{
		var crv = curves.item(i);
		var length = crv.getLength();

		var prev_dist = 0.0;
		var is_gap = false;

		while ( prev_dist < length )
		{
			var next_dist = prev_dist + (is_gap ? GapSize : DashSize);

			if ( !is_gap )
			{
				var p0 = GetPointAtDistance( crv, length, prev_dist );
				var p1 = GetPointAtDistance( crv, length, next_dist );

				var dash = TrimCurve( crv, p0, p1 );
				if ( dash )
					g_Pieces.addObject( dash );
			}

			prev_dist = next_dist;
			is_gap = !is_gap;
		}
	}

	moi.geometryDatabase.addObjects( g_Pieces );

	curves.setProperty( 'hidden', true );
}

function Cancel( curves )
{
	curves.setProperty( 'hidden', false );

	if ( g_Pieces )
	{
		moi.geometryDatabase.removeObjects( g_Pieces );
		g_Pieces = null;
	}
}

function DoDashCurve()
{
	var objectpicker = moi.ui.createObjectPicker();
	objectpicker.allowCurves();
	if ( !GetObjects( objectpicker ) )
		return;
		
	var curves = objectpicker.objects;
	Update( curves );

	var dlg = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			Cancel( curves );
			return;
		}

		if ( dlg.event == 'done' )
			break;

		if ( dlg.event == 'DashSize' || dlg.event == 'GapSize' )
			Update( curves );
	}

	curves.setProperty( 'hidden', false );
	moi.geometryDatabase.removeObjects( curves );
}

DoDashCurve();
